function [u,raw_u] = estimation(theta, other,  seed, reg, rng_type, blks, model, preest)

estclass = model(1); esttype = model(2); eps_dist = model(3); 

T = other(1);
S = other(2);
N = other(3);

if strcmp(preest, 'true')
    if strcmp(esttype, "typeB3") || strcmp(esttype, "typeB4")
    nu    = 1/other(4); 
    lam   = 0;
    gamma = theta(1:end);
    else
    nu    = 1/other(4); 
    lam   = theta(1);
    gamma = theta(2:end);
    end
else
    if strcmp(esttype, "typeB3") || strcmp(esttype, "typeB4")
    nu    = 1/theta(1); 
    lam   = 0;
    gamma = theta(2:end);
    else
    nu    = 1/theta(1); 
    lam   = theta(2);
    gamma = theta(3:end);
    end
end

M    = max(blks);
blSize = nan(M,1);
for i = 1:M
    blSize(i) = sum(blks == i);
end

eps = zeros(S*T,N);

if strcmp(estclass, 'estimable')
    if strcmp(esttype, 'typeB1') || strcmp(esttype, 'typeB3')
        lam0 = lam*(1-1*strcmp(esttype, 'typeB3'));
        F   = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+1,'StreamIndices',1,'Seed',seed+1),T*S,1),nu,lam0);
    elseif strcmp(esttype, 'typeB2') || strcmp(esttype, 'typeB4')
        lam0 = lam*(1-1*strcmp(esttype, 'typeB4'));
        F1  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+4,'StreamIndices',1,'Seed',seed+1),T*S,1),nu,lam);
        F2  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+4,'StreamIndices',2,'Seed',seed+1),T*S,1),nu,lam0);
        F3  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+4,'StreamIndices',3,'Seed',seed+1),T*S,1),nu,lam0);
        F4  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+4,'StreamIndices',4,'Seed',seed+1),T*S,1),nu,lam0);
    end
    kk = 3*(strcmp(esttype, 'typeB2') || strcmp(esttype, 'typeB4'));
    if strcmp(eps_dist, 'norm')
        for i = 1:N
            eps(:,i) = randn(RandStream.create(rng_type,'NumStreams',N+kk+1,'StreamIndices',i+kk+1,'Seed',seed+1),T*S,1);%
        end
    elseif strcmp(eps_dist, 'st')
        for i = 1:N
            eps(:,i) = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+kk+1,'StreamIndices',i+kk+1,'Seed',seed+1),T*S,1),nu,0);
        end
    end
elseif strcmp(estclass, 'benchmark')
    if strcmp(esttype, 'typeA1') || strcmp(esttype, 'typeA2')
        F   = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+1,'StreamIndices',1,'Seed',seed+1),T*S,1),nu,lam);
    elseif strcmp(esttype, 'typeA3') 
        F   = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+2,'StreamIndices',1,'Seed',seed+1),T*S,1),nu,lam);
        F1  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+2,'StreamIndices',2,'Seed',seed+1),T*S,1),nu,0);
    elseif strcmp(esttype, 'typeA4')
        F   = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+5,'StreamIndices',1,'Seed',seed+1),T*S,1),nu,lam);
        F1  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+5,'StreamIndices',2,'Seed',seed+1),T*S,1),nu,0);
        F2  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+5,'StreamIndices',3,'Seed',seed+1),T*S,1),nu,0);
        F3  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+5,'StreamIndices',4,'Seed',seed+1),T*S,1),nu,0);
        F4  = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+5,'StreamIndices',5,'Seed',seed+1),T*S,1),nu,0);
    end
    kk = 1*strcmp(esttype, 'typeA3')+4*strcmp(esttype, 'typeA4');
    if strcmp(eps_dist, 'norm')
        for i = 1:N
            eps(:,i) = randn(RandStream.create(rng_type,'NumStreams',N+kk+1,'StreamIndices',i+kk+1,'Seed',seed+1),T*S,1);%
        end
    elseif strcmp(eps_dist, 'st')
        for i = 1:N
            eps(:,i) = skewtdis_inv2(rand(RandStream.create(rng_type,'NumStreams',N+kk+1,'StreamIndices',i+kk+1,'Seed',seed+1),T*S,1),nu,0);
        end
    end
end

if strcmp(estclass, 'benchmark')
    if strcmp(esttype, 'typeA1')
        raw_u =   F*ones(1,N).*gamma(1)  + eps;
    elseif strcmp(esttype, 'typeA2')
        raw_u =   [(F*ones(1,blSize(1))).*gamma(1)  (F*ones(1,blSize(2))).*gamma(2) (F*ones(1,blSize(3))).*gamma(3) (F*ones(1,blSize(4))).*gamma(4)]  + eps;%
    elseif strcmp(esttype, 'typeA3')
        raw_u =    F*ones(1,N).*gamma(1)  +...
            [(F1*ones(1,blSize(1))).*gamma(2)  (F1*ones(1,blSize(2))).*gamma(3)...
            (F1*ones(1,blSize(3))).*gamma(4) (F1*ones(1,blSize(4))).*gamma(5)]  + eps;
    elseif strcmp(esttype, 'typeA4')
        raw_u = [(F*ones(1,blSize(1))).*gamma(1) (F*ones(1,blSize(2))).*gamma(2)...
            (F*ones(1,blSize(3))).*gamma(3) (F*ones(1,blSize(4))).*gamma(4)] +...
            [(F1*ones(1,blSize(1))).*gamma(5) (F2*ones(1,blSize(2))).*gamma(6)...
            (F3*ones(1,blSize(3))).*gamma(7) (F4*ones(1,blSize(4))).*gamma(8)] + eps;
    end
elseif strcmp(estclass, 'estimable')
    Z = repmat(reg,S,1);
    if strcmp(esttype, 'typeB1')
        raw_u =   [(F*ones(1,blSize(1))).*gamma(2)  (F*ones(1,blSize(2))).*gamma(3) ...
                   (F*ones(1,blSize(3))).*gamma(4)  (F*ones(1,blSize(4))).*gamma(5)] +  Z*ones(1,N).*gamma(1) + eps;
    elseif strcmp(esttype, 'typeB2')
        raw_u =   [(F1*ones(1,blSize(1))).*gamma(2)  (F2*ones(1,blSize(2))).*gamma(3) ...
                   (F3*ones(1,blSize(3))).*gamma(4)  (F4*ones(1,blSize(4))).*gamma(5)] +  Z*ones(1,N).*gamma(1) + eps;
    elseif strcmp(esttype, 'typeB3') 
        raw_u = [(Z*ones(1,blSize(1))).*gamma(1) (Z*ones(1,blSize(2))).*gamma(2)...
            (Z*ones(1,blSize(3))).*gamma(3) (Z*ones(1,blSize(4))).*gamma(4)] +...
            [(F*ones(1,blSize(1))).*gamma(5) (F*ones(1,blSize(2))).*gamma(6)...
            (F*ones(1,blSize(3))).*gamma(7) (F*ones(1,blSize(4))).*gamma(8)] + eps;
    elseif strcmp(esttype, 'typeB4')
        raw_u = [(Z*ones(1,blSize(1))).*gamma(1) (Z*ones(1,blSize(2))).*gamma(2)...
                 (Z*ones(1,blSize(3))).*gamma(3) (Z*ones(1,blSize(4))).*gamma(4)] +...
                [(F1*ones(1,blSize(1))).*gamma(5) (F2*ones(1,blSize(2))).*gamma(6)...
                 (F3*ones(1,blSize(3))).*gamma(7) (F4*ones(1,blSize(4))).*gamma(8)] + eps;
    end
end

u  = tiedrank(raw_u)./(S*T+1);
end
