function mrkcorr = mSrho(x,y) 
% function mrkcorr = mSrho(x,y) 
%
% Function to compute rank correlation between x and y
%
% INPUTS: x, a Tx1 vector of data
%         y, a Tx1 vector of data
%
% OUTPUTS: mrkcorr, a scalar, the rank correlation between x and y
%
%  Dong Hwan Oh and Andrew J. Patton
%
%  22 October 2013
%
% This code was developed for the following paper:
%
% Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
% for Copula-Based Multivariate Models," Journal of the American
% Statistical Association, 108(502), 689-700.


[T N] = size(x) ;

mean_x = sum(x)/T ;
mean_y = sum(y)/T ;

numer  = (x - mean_x).*(y - mean_y) ;
deno   = sqrt( (sum((x - mean_x).^2)/T)* ...
               (sum((y - mean_y).^2)/T) ) ;
           
 mrkcorr = numer/deno ;

 