function out = mQdep(x,y,q) 
% function out = mQdep(x,y,q) 
%
% Function to compute quantile dependence between x and y
%
% INPUTS: x, a Tx1 vector of data
%         y, a Tx1 vector of data
%         q, a scalar in (0,1)
%
% OUTPUTS: out, a scalar, the quantile dependence between x and y
%
%  Dong Hwan Oh and Andrew J. Patton
%
%  22 October 2013
%
% This code was developed for the following paper:
%
% Oh, D.H. and A.J. Patton, 2013, "Simulated Method of Moments Estimation
% for Copula-Based Multivariate Models," Journal of the American
% Statistical Association, 108(502), 689-700.


if q<0.5
    out = ( (x<=q).*(y<=q) ) /q ;
else 
    out = ( (x>q).*(y>q) ) /(1-q); 
end
