function [x, y, x_opt, y_opt] = simulate_copula(theta0,q,LB,UB,T,S,S1,N,B,model,ar,test,rhoS,rng_type,ini,solver,Seed)
% block-equi factor copula with Q = 3, |G_q| = 15, q = 1,2,3.
% x = SMM estimate 
% y = 1 if t-stats/J-stats reject 0 otherwise 
% x_opt,y_opt same as x and y but based on optimal weight matrix

disp(strcat('Round:',num2str(Seed)))   
seed     = 123*Seed;   
seed_est = 456*Seed;

Weight     = 1;
blks = [ones(N/3,1)' repmat(2, N/3,1)' repmat(3, N/3,1)']';

other_null = [T N];
other_est  = [T S N];
phi        = [0.01, 0.05, 0.05, 0.10, 0.85];

% some container
data_resid=NaN(T,N); % AR-GARCH residuals
data_param=NaN(N,length(phi)); % AR-GARCH parameters
data_uresid_semipar=NaN(T,N); % ARMA-GARCH PIT residuals (semiparametric)

[Z, Z_hat] = reg_generate(other_null, ar,  model, rng_type, seed);

[data, ~] = return_dgp(theta0,other_null,phi,Z,model,seed,rng_type,blks);     
   
for nn=1:N
                [~,temp_param,~,~,temp_resid,~]=fitARMAGARCH(data(:,nn),[1,0,1,0,1],'NORMAL',phi);
                data_resid(:,nn)=temp_resid;
                data_param(nn,:)=temp_param;
                data_uresid_semipar(:,nn)=tiedrank(temp_resid)/(T+1);
end 
clear temp_resid temp_param


fun  = @(x)  objfunc(x, other_est,  data_uresid_semipar, Weight, q, Z_hat, model,seed_est, blks, rng_type,rhoS,'false');
fun1 = @(x)  objfunc(x, [T S1 N],   data_uresid_semipar, Weight, q, Z_hat, model,seed_est, blks, rng_type,rhoS,'false');
fun2 = @(x)  ini_grid(x,data_uresid_semipar,Weight,q,Z_hat,model,blks,LB,UB,T,N, S1,rhoS,rng_type,ini,seed_est);

opts = optimoptions('surrogateopt','UseParallel',false,'PlotFcn',[],'display','off'); 

[theta_ini1,Q1,~,~] = surrogateopt(fun1,LB,UB, opts);                        % obtain raw initial value using surrogateopt() w/o need for starting values
[theta_ini2,Q2]     = fun2(theta_ini1);  

if Q2 < Q1
    theta_ini = theta_ini2;
else
    theta_ini = theta_ini1;
end

if strcmp(solver, 'pattern')
    opts1 =  optimset('display','off');llb = []; uub = [];
    [theta_hat1,Q1,~,~] = fminsearchbnd(fun, theta_ini,LB,UB);
    [theta_hat2,Q2,~,~] = patternsearch(fun, theta_hat1,[],[],llb,uub,LB,UB,[],opts1);
    if Q1 < Q2
        theta_hat = theta_hat1;
    else
        theta_hat = theta_hat2;
    end
elseif strcmp(solver, 'fmin')
    [theta_ini3,Q3,~,~] = fminsearchbnd(fun, theta_ini,LB,UB);
    theta_hat = theta_ini3;
end


x = [theta_hat]
 
%%
 
if(test == 1)   
[~,Xsim]  = estimation(theta_hat,other_est,Z_hat, model,seed_est,rng_type,blks, 'false');
Sigma_hat = var_est(data_resid,Xsim,q,B,seed_est,blks,rhoS);  
clear Xsim
[J_hat05,~] = num_deriv(theta_hat,data_uresid_semipar,other_est,Weight,blks,q, Z_hat, model,seed_est,0.05,LB,UB,rng_type,rhoS);
[J_hat01,~] = num_deriv(theta_hat,data_uresid_semipar,other_est,Weight,blks,q, Z_hat, model,seed_est,0.01,LB,UB,rng_type,rhoS);
Omega_hat_05   = (J_hat05'*Weight*J_hat05)\(J_hat05'*Weight*Sigma_hat*Weight*J_hat05)/(J_hat05'*Weight*J_hat05);
Omega_hat_01   = (J_hat01'*Weight*J_hat01)\(J_hat01'*Weight*Sigma_hat*Weight*J_hat01)/(J_hat01'*Weight*J_hat01);
[~,ghat] = objfunc(theta_hat,other_est,data_uresid_semipar,Weight,q,Z_hat,model,seed_est,blks,rng_type,rhoS, 'false');
[J05, ~,~,~] = JTest(ghat,T,1000,Weight,Sigma_hat,J_hat05,seed_est);
[J01, ~,~,~] = JTest(ghat,T,1000,Weight,Sigma_hat,J_hat01,seed_est);


test05 = sqrt(T)*(theta_hat-theta0)./sqrt(diag(Omega_hat_05))';
test01 = sqrt(T)*(theta_hat-theta0)./sqrt(diag(Omega_hat_01))';

 
y = abs([test05 test01]) > 1.96;

y = [y J05 J01];            

[x1, y1, z1] = smm_opt(Sigma_hat,other_est,theta_hat, theta0, data_uresid_semipar, q, Z_hat, seed, rng_type, rhoS, model, blks, LB, UB);

x_opt = x1;
y_opt = [abs(y1)> 1.96 z1];
else
    y = NaN;
    x_opt = nan(length(x),1);
    y_opt = nan(length(y),1);
end

end



 


